// The using section is strictly ordered by name
// Part I: GNavigia items
using KorKarNet.GNavigia;
using KorKarNet.Utilities;
using KorKarNet.Windows.Utilities;
// Part II: System items
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;

namespace GNavigiaGpsElementsClient
{
	/// <summary>
	/// Class <b>GNavigiaGpsElementsClient</b> is designed to demonstrate the programming of GNavigia.
	/// </summary>
	public class GNavigiaGpsElementsClient : IGpsElementClient
	{
		#region Logger
		protected static readonly log4net.ILog log
			= log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
		#endregion

		#region Variables and constructor(s)
		// The client name, human readable.
		private const string cs_clientName = "GNavigiaGpsElementsClient";
		// The client id, COM readable, unique.
		private const string cs_clientID = "96C97E05-4B2C-4656-BD35-A1D4AF9E71B7";
		// All other class variables go here.
		private IGpsElementServer m_gpsElementServer;

		/// <summary>
		/// The constructor is essential. Without this, the Server cannot establish
		/// a connection to the client.
		/// </summary>
		public GNavigiaGpsElementsClient()
		{
			log.InfoFormat("** {0}: Startup", cs_clientName);
		}
		#endregion

		#region IGpsElementClient members, explicit implementation

		bool IGpsElementClient.Inititialize(IGpsElementServer gpsElementServer)
		{
			log.InfoFormat("** {0}: {1}", cs_clientName, MethodBase.GetCurrentMethod().Name);
			m_gpsElementServer = gpsElementServer;
			return true;
		}

		MenuItem IGpsElementClient.GetMenu(ref bool fAddToMainMenu)
		{
			log.InfoFormat("** {0}: {1}", cs_clientName, MethodBase.GetCurrentMethod().Name);
			List<MenuItem> list = new List<MenuItem>();
			EventHandler eventHandler = new EventHandler(OnMenuItemClicked);
			MenuItem menuItem = new MenuItem("&Wegpunktdatumskorrektur", eventHandler);
			menuItem.Name = "WDK";
			list.Add(menuItem);
			menuItem = new MenuItem("&Lnge ausgewhlter Tracks", eventHandler);
			menuItem.Name = "LAT";
			list.Add(menuItem);
			menuItem = new MenuItem("Menpunkt &3", eventHandler);
			menuItem.Name = "MP3";
			list.Add(menuItem);
			// The clients root menu.
			MenuItem menu = new MenuItem("&Client", list.ToArray());
			// Add handler here
			menu.Popup += new EventHandler(OnMenuPopup);
#if MakeSubmenu
			fAddToMainMenu = false;
#endif
			return menu;
		}

		void IGpsElementClient.OnPaint(GpsDataEditor gpsDataEditor,
									   GpsDataEditor.GpsGraphics gpsGraphics, bool fOnBeginPaint)
		{
			if (!fOnBeginPaint)	// To draw on top of all!
			{
			}
		}

		bool IGpsElementClient.OnReadData(GpsDataEditor gpsDataEditor, string clientName,
										  string innerXml)
		{
			log.InfoFormat("** {0}: {1}", cs_clientName, MethodBase.GetCurrentMethod().Name);
			if (clientName == cs_clientID)
			{	// analyze data string here ...
				return true;
			}
			return false;
		}

		string IGpsElementClient.OnWriteData(GpsDataEditor gpsDataEditor, out string clientName)
		{
			log.InfoFormat("** {0}: {1}", cs_clientName, MethodBase.GetCurrentMethod().Name);
			clientName = cs_clientID;
			return "<Testdaten>Testdaten</Testdaten>";
		}
		#endregion

		#region Private methods
		void OnMenuItemClicked(object sender, EventArgs e)
		{
			MenuItem menuItem = sender as MenuItem;
			switch (menuItem.Name)
			{
				case "WDK":
					int count = 0;
					foreach (GpsWaypoint gpsWaypoint in m_gpsElementServer.GpsWaypoints())
						try
						{	// If attributes desc and cmt have the same value, we parse
							// the special date string and set it to DateAndTime.
							// We expect parse errors, so we catch exceptions here.
							string date = gpsWaypoint.Desc;
							if (date != null && date == gpsWaypoint.Cmt)
							{
								string[] sl = date.Split("- ".ToCharArray());
								if (sl != null && sl.Length == 4)
								{
									gpsWaypoint.DateTime = DateTime.Parse(date);
									if (Str.IsInt(gpsWaypoint.Name))
									{	// If someone has already altered the name, do not touch it!
										gpsWaypoint.Cmt = gpsWaypoint.Name;
										gpsWaypoint.Desc = null;
									}
									count++;
								}
							}
						}
						catch (Exception ex) { log.Error(ex.Message); }
					MessageBox.Show(m_gpsElementServer as IWin32Window,
						String.Format("Anzahl der Wegpunkte:  {0,3}" + Environment.NewLine
							+ "Anzahl der nderungen: {1,3}", m_gpsElementServer.GpsWaypoints().Count, count),
							cs_clientName, MessageBoxButtons.OK, MessageBoxIcon.Information);
					break;

				default:
					MessageBox.Show(m_gpsElementServer as IWin32Window,
									"Dieser Menpunkt ist noch nicht implementiert!");
					break;
			}
		}

		void OnMenuPopup(object sender, EventArgs e)
		{
			foreach (MenuItem menuItem in (sender as MenuItem).MenuItems)
				switch (menuItem.Name)
				{
					case "WDK":
						menuItem.Enabled = m_gpsElementServer.GpsWaypoints().Count > 0;
						break;
					default:
						menuItem.Enabled = true;
						break;
				}
		}
		#endregion
	}
}
